#include <stdio.h>
#include <stdint.h>
#include <string.h>
#include <expat.h>

#include "common.h"
#include "locodef.h"
#include "objects.h"

void putxmlstr(FILE *f, const char *buffer, uint32_t minlen)
{
  uint32_t i;

  if (!minlen) minlen = strlen(buffer);

  for (i=0; i<minlen; i++)
	switch (buffer[i]) {
		case '"': fputs("&quot;", f); break;
		case '<': fputs("&lt;", f); break;
		case '>': fputs("&gt;", f); break;
		case '&': fputs("&amp;", f); break;
//		case 0 ... 31: fprintf(f, "&#x%02X;", buffer[i]); break;
		default: fputc(buffer[i], f); break;
	}
}

static void XMLCALL startElement(void *userData, const char *name, const char **atts)
{
  int i;
  int *depthPtr = userData;
  for (i = 0; i < *depthPtr; i++)
    putchar('\t');
  puts(name);
  *depthPtr += 1;
}

static void XMLCALL endElement(void *userData, const char *name)
{
  int *depthPtr = userData;
  *depthPtr -= 1;
}


uint32_t parsexml(uint8_t *data, uint32_t len, uint8_t **outdata)
{
  XML_Parser parser;
  struct xmlinf *parsedinfo;

  parsedinfo = my_alloc(sizeof(parsedinfo), "parsedinfo");

  parser = XML_ParserCreate(NULL);
  XML_SetUserData(parser, parsedinfo);
  XML_SetElementHandler(parser, startElement, endElement);
  if (XML_Parse(parser, data, len, 1) == XML_STATUS_ERROR)
	die("Error while parsing xml: %s at line %d",
	      XML_ErrorString(XML_GetErrorCode(parser)),
	      XML_GetCurrentLineNumber(parser));
  XML_ParserFree(parser);

  return parseprocs[parsedinfo->class](parsedinfo, outdata);
}
